﻿using DinerContracts.BindingModels;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using DinerDataBaseImplement.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DinerDataBaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new DinerDatabase();

            return context.Orders
                    .Include(x => x.Snack)
                    .Select(x => x.GetViewModel)
                    .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && !model.DateFrom.HasValue && !model.DateTo.HasValue)
            {
                return new();
            }
            using var context = new DinerDatabase();
            if (!model.Id.HasValue && model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                return context.Orders
                    .Include(x => x.Snack)
                    .Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                    .Select(x => x.GetViewModel)
                    .ToList();
            }
            return context.Orders
                .Include(x => x.Snack)
                .Where(x => x.Id == model.Id)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new DinerDatabase();
            return context.Orders.Include(x => x.Snack)
             .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return context.Orders.Include(x => x.Snack)
                                 .FirstOrDefault(x => x.Id == newOrder.Id)?.GetViewModel; ;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
                var order = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
                if (order == null)
                {
                    return null;
                }
                order.Update(model);
                context.SaveChanges();

                return context.Orders
                    .Include(x => x.Snack)
                    .FirstOrDefault(x => x.Id == model.Id)?
                    .GetViewModel;
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new DinerDatabase();
            var element = context.Orders
            .FirstOrDefault(x => x.Id == model.Id);

            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
