﻿using DinerContracts.BindingModels;
using DinerContracts.SearchModels;
using DinerContracts.StoragesContracts;
using DinerContracts.ViewModels;
using DinerFileImplement.Models;

namespace DinerFileImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        private readonly DataFileSingleton source;
        public OrderStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            var element = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Orders.Remove(element);
                source.SaveOrders();

                return AttachSnackName(element.GetViewModel);
            }
            return null;
        }

        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            return AttachSnackName(source.Orders.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel);
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue && model.DateFrom.HasValue && model.DateTo.HasValue)
            {
                return source.Orders.Where(x => x.DateCreate >= model.DateFrom && x.DateCreate <= model.DateTo)
                   .Select(x => AttachSnackName(x.GetViewModel))
                   .ToList();
            }
            return source.Orders.Where(x => x.Id == model.Id).Select(x => AttachSnackName(x.GetViewModel)).ToList();
        }

        public List<OrderViewModel> GetFullList()
        {
            return source.Orders
                .Select(x => AttachSnackName(x.GetViewModel)).ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            model.Id = source.Orders.Count > 0 ? source.Orders.Max(x => x.Id) + 1 : 1;
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            source.Orders.Add(newOrder);
            source.SaveOrders();
            return AttachSnackName(newOrder.GetViewModel);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            var order = source.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            source.SaveOrders();
            return AttachSnackName(order.GetViewModel);
        }
        private OrderViewModel? AttachSnackName(OrderViewModel? model)
        {
            if (model == null)
            {
                return null;
            }
            var snack = source.Snacks.FirstOrDefault(x => x.Id == model.SnackId);
            if (snack != null)
            {
                model.SnackName = snack.SnackName;
            }
            return model;
        }
    }
}
